import React, {
  useState,
  useCallback,
  useEffect,
} from 'react';
import ReactDOM from 'react-dom';
import {
  csv,
  scaleLinear,
  min,
  max,
  extent,
  format,
} from 'd3';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';
import { Dropdown } from './Dropdown';

const menuHeight = 75;
const width = 960;
const height = 500 - menuHeight;
const margin = {
  top: 20,
  right: 25,
  bottom: 75,
  left: 100,
};
const attributes = [
  { value: 'sepal_length', label: 'Sepal Length' },
  { value: 'sepal_width', label: 'Sepal Width' },
  { value: 'petal_length', label: 'Petal Length' },
  { value: 'petal_width', label: 'Petal Width' },
  { value: 'species', label: 'Species' },
];

// loop thru attributes value, if matches, return label
const getLabel = (value) => {
  for (let i = 0; i < attributes.length; i++) {
    if (attributes[i].value === value){
    	return attributes[i].label;
    }
    
  }
};
const App = () => {
  const initialXAttribute = 'petal_length';
  const [xAttribute, setXAttribute] = useState(
    initialXAttribute
  );

  const xValue = (d) => d[xAttribute];
  const xAxisLabel = getLabel(xAttribute);
  const xAxisLabelOffset = 60;

  const initialYAttribute = 'sepal_width';
  const [yAttribute, setYAttribute] = useState(
    initialYAttribute
  );

  const yValue = (d) => d[yAttribute];
  const yAxisLabel = getLabel(yAttribute);
  const yAxisLabelOffset = 55;

  const siFormat = format('.2s');
  // we move the si format to its own thing
  // because it would keep invoking / creating otherwise?
  const xAxisTickFormat = (tickValue) =>
    siFormat(tickValue).replace('G', 'B');

  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;
  const data = useData();

  if (!data) {
    return <pre>Loading...</pre>;
  }

  //console.log(data[0]);

  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([0, innerHeight]);

  return (
    <>
      <label for="x-select">X:</label>
      <Dropdown
        options={attributes}
        id="x-select"
        selectedValue={xAttribute}
        onSelectedValueChange={setXAttribute}
      />
      <label for="y-select">Y:</label>
      <Dropdown
        options={attributes}
        id="y-select"
        selectedValue={yAttribute}
        onSelectedValueChange={setYAttribute}
      />
      <svg width={width} height={height}>
        <g
          transform={`translate(${margin.left}, ${margin.top})`}
        >
          <AxisBottom
            xScale={xScale}
            innerHeight={innerHeight}
            tickFormat={xAxisTickFormat}
            tickOffset={5}
          />
          <text
            className="axis-label"
            text-anchor="middle"
            transform={`translate(${-yAxisLabelOffset},${
              innerHeight / 2
            }) rotate(-90) `}
          >
            {yAxisLabel}
          </text>
          <AxisLeft
            yScale={yScale}
            innerWidth={innerWidth}
            tickOffset={8}
          />
          <text
            className="axis-label"
            x={innerWidth / 2}
            text-anchor="middle"
            y={innerHeight + xAxisLabelOffset}
          >
            {xAxisLabel}
          </text>
          <Marks
            data={data}
            xScale={xScale}
            yScale={yScale}
            xValue={xValue}
            yValue={yValue}
            tooltipFormat={xAxisTickFormat}
            circleRadius={7}
          />
        </g>
      </svg>
    </>
  );
};
const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
